--[[

SurfaceControl, controls what Surface-1.0 and SharedMedia-1.0 returns from the Fetch method.
Copyright (C) 2007 rabbit.magtheridon@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

]]

local surface = AceLibrary:HasInstance("Surface-1.0") and AceLibrary("Surface-1.0") or nil
local sharedMedia = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or nil
local sm2 = AceLibrary:HasInstance("LibSharedMedia-2.0") and AceLibrary("LibSharedMedia-2.0") or nil
local sm3 = AceLibrary:HasInstance("LibSharedMedia-3.0") and AceLibrary("LibSharedMedia-3.0") or nil
local sink = AceLibrary:HasInstance("Sink-1.0") and AceLibrary("Sink-1.0") or nil
if not surface and not sharedMedia and not sink and not sm2 then return end

SurfaceControl = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

local sharedMediaOverrideTypes = {
	"statusbar"
}

function SurfaceControl:OnInitialize()
	self:RegisterDB("SurfaceControlDB")
	self:RegisterDefaults("profile", {
		texture = nil,
		sink = nil,
	})
	self:RegisterChatCommand("/surfacecontrol", {
		type = "group",
		name = "Surface Control",
		desc = "Change override settings for Surface-1.0, SharedMedia-1/2/3 and Sink-1.0.",
		args = {
			texture = {
				type = "text",
				name = "Texture",
				desc = "Set the override texture for Surface-1.0 and SharedMedia-1/2/3's overridable types.",
				usage = "<texture>",
				get = function() return self.db.profile.texture end,
				set = function(v)
					self.db.profile.texture = v
					self:SetSharedMediaGlobal()
					self:SetSurfaceGlobal()
					self:SetSharedMedia2Global()
					self:SetSharedMedia3Global()
				end,
				validate = sm3 and sm3:List("statusbar") or sm2 and sm2:List("statusbar") or sharedMedia and sharedMedia:List("statusbar") or surface and surface:List(),
			},
			sink = {
				type = "text",
				name = "Sink",
				desc = "Set the override output sink for Sink-1.0",
				usage = "<sink>",
				get = function() return self.db.profile.sink end,
				set = function(v)
					self.db.profile.sink = v
					self:SetSink10Override()
				end,
				validate = {"Parrot", "SCT", "MikSBT", "BigWigs", "ChatFrame", "Blizzard", "RaidWarning", "None"},
			},
		},
	}, "SURFACECONTROL")
end

function SurfaceControl:OnEnable()
	self:RegisterEvent("AceEvent_FullyInitialized")
	self:RegisterEvent("AceLibrary_Register")

	if Rock and Rock:HasLibrary("LibRockEvent-1.0") then
		Rock("LibRockEvent-1.0"):Embed(self)
		self:AddEventListener("LibRock-1.0", "LibraryLoad", "AceLibrary_Register")
	end

	if AceLibrary("AceEvent-2.0"):IsFullyInitialized() then
		self:AceEvent_FullyInitialized()
	end
end

function SurfaceControl:AceEvent_FullyInitialized()
	self:SetSurfaceGlobal()
	self:SetSharedMediaGlobal()
	self:SetSharedMedia2Global()
	self:SetSharedMedia3Global()
	self:SetSink10Override()

	self:UpdateAddons()
end

function SurfaceControl:AceLibrary_Register(major, instance)
	local f = nil
	if major == "Surface-1.0" then
		self:SetSurfaceGlobal(instance)
		f = true
	elseif major == "SharedMedia-1.0" then
		self:SetSharedMediaGlobal(instance)
		f = true
	elseif major == "LibSharedMedia-2.0" then
		self:SetSharedMedia2Global(instance)
		f = true
	elseif major == "LibSharedMedia-3.0" then
		self:SetSharedMedia3Global(instance)
		f = true
	elseif major == "Sink-1.0" then
		self:SetSink10Override(instance)
		f = true
	end
	if f then self:UpdateAddons() end
end

function SurfaceControl:SetSink10Override(instance)
	if not instance then instance = sink end
	if not instance or type(instance.SetSinkOverride) ~= "function" then return end
	local overrideSink = self.db.profile.sink
	if type(overrideSink) ~= "string" then return end

	instance:SetSinkOverride(overrideSink)
end

function SurfaceControl:SetSurfaceGlobal(instance)
	if not instance then instance = surface end
	if not instance or type(instance.SetGlobal) ~= "function" or type(instance.IsValid) ~= "function" then return end
	local texture = self.db.profile.texture
	if type(texture) ~= "string" or not instance:IsValid(texture) then return end

	instance:SetGlobal(texture)
end

function SurfaceControl:SetSharedMediaGlobal(instance)
	if not instance then instance = sharedMedia end
	if not instance or type(instance.SetGlobal) ~= "function" or type(instance.IsValid) ~= "function" then return end
	local texture = self.db.profile.texture
	if type(texture) ~= "string" then return end

	for i, v in ipairs(sharedMediaOverrideTypes) do
		if instance:IsValid(v, texture) then
			instance:SetGlobal(v, texture)
		end
	end
end

function SurfaceControl:SetSharedMedia2Global(instance)
	if not instance then instance = sm2 end
	if not instance or type(instance.SetGlobal) ~= "function" or type(instance.IsValid) ~= "function" then return end
	local texture = self.db.profile.texture
	if type(texture) ~= "string" then return end

	for i, v in ipairs(sharedMediaOverrideTypes) do
		if instance:IsValid(v, texture) then
			instance:SetGlobal(v, texture)
		end
	end
end

function SurfaceControl:SetSharedMedia3Global(instance)
	if not instance then instance = sm3 end
	if not instance or type(instance.SetGlobal) ~= "function" or type(instance.IsValid) ~= "function" then return end
	local texture = self.db.profile.texture
	if type(texture) ~= "string" then return end

	for i, v in ipairs(sharedMediaOverrideTypes) do
		if instance:IsValid(v, texture) then
			instance:SetGlobal(v, texture)
		end
	end
end

function SurfaceControl:UpdateAddons()
	-- Any addons that need to be reset
	if type(aUF) == "table" and type(aUF.CallUnitMethods) == "function" then
		aUF:CallUnitMethods("BarTexture")
	end

	if type(CBRipoff) == "table" and type(CBRipoff.Layout) == "function" then
		CBRipoff:Layout("CastingBar")
		CBRipoff:Layout("BREATH", "MirrorBar")
		CBRipoff:Layout("EXHAUSTION", "MirrorBar")
		CBRipoff:Layout("FEIGNDEATH", "MirrorBar")
	end

	if type(PitBull) == "table" and type(PitBull.UpdateAllStatusBarTextures) == "function" then
		PitBull:UpdateAllStatusBarTextures()
	end
end
